"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const app_1 = require("./app");
const app_configuration_1 = require("./app-configuration");
const logger_1 = require("./logger");
const viewFunctionSource = {
    drawer: {
        items(...items) {
            this._settings.drawerItems = items;
        },
        activeItem(name) {
            this._settings.drawerActiveItem = name;
        },
        disableItems(...items) {
            this._settings.drawerItemsDisabled = items;
        },
    },
    public(isPublic = true) {
        this._settings.isPublic = isPublic;
    },
    theme(color) {
        this._settings.theme = color;
    }
};
const appFunctionSource = {
    title(titleName) {
        this._settings.title = titleName;
    },
    routesFile(filePath) {
        this._settings.routesConfigFile = filePath;
    },
    settingsFile(filePath) {
        this._settings.settingsFile = filePath;
    },
    settings(options) {
        this._settings.configuration = options;
    },
    routes(routes) {
        this._settings.routes = routes;
    }
};
const findSourceFunctions = function (configParent, objectParent) {
    let functionCollection = new Map();
    for (const key of Object.keys(configParent)) {
        const property = configParent[key];
        if (typeof property !== "object" || property instanceof Array) {
            functionCollection.set(objectParent[key], [property]);
            continue;
        }
        functionCollection = new Map([
            ...functionCollection,
            ...findSourceFunctions(property, objectParent[key])
        ]);
    }
    return functionCollection;
};
exports.onlyIf = function (callback, elseCallbackName) {
    return function (target, propertyKey, descriptor) {
        const fOldValue = descriptor.value;
        const sElseCallback = elseCallbackName || propertyKey + "Else";
        descriptor.value = function () {
            if (!callback.call(this, this, propertyKey)) {
                this[sElseCallback] && this[sElseCallback].call(this);
                return;
            }
            return fOldValue.apply(this, arguments);
        };
        return descriptor;
    };
};
exports.onlyIf.hasParameters = function (...args) {
    return function (target, propertyKey, descriptor) {
        const fOldValue = descriptor.value;
        const sValue = fOldValue.toLocaleString();
        const aParamsNames = sValue.substring(sValue.indexOf("(") + 1, sValue.indexOf(")")).split(", ");
        const params = {};
        args.forEach((sParamName) => {
            params[sParamName] = aParamsNames.indexOf(sParamName);
        });
        if (!Object.keys(params).length) {
            logger_1.log.warn(`{${target.constructor.name}} ${propertyKey}()`, `No matching parameter found. Decorator will have no effect`);
        }
        descriptor.value = function () {
            let bAbort = false;
            for (const key in params) {
                if (arguments[params[key]] !== undefined) {
                    continue;
                }
                logger_1.log.error(`{${target.constructor.name}} ${propertyKey}()`, `Parameter '${key}' has not been defined`);
                bAbort = true;
            }
            if (bAbort) {
                return;
            }
            return fOldValue.apply(this, arguments);
        };
        return descriptor;
    };
};
exports.onlyIf.hasProperties = function (...args) {
    return function (target, propertyKey, descriptor) {
        const fOldValue = descriptor.value;
        descriptor.value = function () {
            let bAbort = false;
            args.forEach((arg) => {
                if (this[arg] !== undefined) {
                    return;
                }
                logger_1.log.error(`{${target.constructor.name}} ${propertyKey}()`, `'${arg}' is not defined`);
                bAbort = true;
            });
            if (bAbort) {
                return;
            }
            return fOldValue.apply(this, arguments);
        };
        return descriptor;
    };
};
exports.onlyIf.isOnline = function (target, propertyKey, descriptor) {
    const fOldValue = descriptor.value;
    const elseCallback = descriptor.offlineCallbackName || `${propertyKey}Offline`;
    descriptor.value = function () {
        if (!navigator.onLine) {
            this[elseCallback] && this[elseCallback].call(this);
            return;
        }
        return fOldValue.apply(this, arguments);
    };
    return descriptor;
};
exports.onlyIf.underDevelopment = function (target, propertyKey, descriptor) {
    const fOldValue = descriptor.value;
    descriptor.value = function () {
        if (!app_configuration_1.PitayaAppConfiguration.Development.enabled) {
            return;
        }
        return fOldValue.apply(this);
    };
    return descriptor;
};
exports.view = function (config) {
    const functionsMap = findSourceFunctions(config, viewFunctionSource);
    return function (constructor) {
        return class extends constructor {
            constructor() {
                super();
                for (const [decorator, paramsCollection] of functionsMap) {
                    decorator.call(this, ...paramsCollection.flat());
                }
            }
        };
    };
};
exports.view.drawer = function (config) {
    const functionsMap = findSourceFunctions(config, viewFunctionSource.drawer);
    return function (constructor) {
        return class extends constructor {
            constructor(...args) {
                super(...args);
                for (const [decorator, paramsCollection] of functionsMap) {
                    decorator.call(this, ...paramsCollection.flat());
                }
            }
        };
    };
};
exports.view.drawer.items = function (...items) {
    return function (constructor) {
        return class extends constructor {
            constructor(...args) {
                super(...args);
                viewFunctionSource.drawer.items.call(this, ...items);
            }
        };
    };
};
exports.view.drawer.disableItems = function (...items) {
    return function (constructor) {
        return class extends constructor {
            constructor(...args) {
                super(...args);
                viewFunctionSource.drawer.disableItems.call(this, ...items);
            }
        };
    };
};
exports.view.drawer.activeItem = function (name) {
    return function (constructor) {
        return class extends constructor {
            constructor(...args) {
                super(...args);
                viewFunctionSource.drawer.activeItem.call(this, name);
            }
        };
    };
};
exports.view.redirectTo = function (route, condition) {
    return function (constructor) {
        return class extends constructor {
            constructor(...args) {
                super(...args);
                condition.call(this, this, route || "") && app_1.PitayaApp.instance.navigate(route || "");
                return;
            }
        };
    };
};
exports.view.public = function (constructor) {
    return class extends constructor {
        constructor(...args) {
            super(...args);
            viewFunctionSource.public.call(this, true);
        }
    };
};
exports.view.theme = function (color) {
    return function (constructor) {
        return class extends constructor {
            constructor(...args) {
                super(...args);
                viewFunctionSource.theme.call(this, color);
            }
        };
    };
};
exports.app = function (config) {
    const functionsMap = findSourceFunctions(config, appFunctionSource);
    return function (constructor) {
        return class extends constructor {
            constructor() {
                super();
                for (const [decorator, paramsCollection] of functionsMap) {
                    decorator.call(this, ...paramsCollection.flat());
                }
            }
        };
    };
};
exports.app.title = function (titleName) {
    return function (constructor) {
        return class extends constructor {
            constructor(...args) {
                super(...args);
                appFunctionSource.title.call(this, titleName);
            }
        };
    };
};
exports.app.configure = function (options) {
    return function (constructor) {
        return class extends constructor {
            constructor(...args) {
                super(...args);
                appFunctionSource.settings.call(this, options);
            }
        };
    };
};
exports.app.routes = function (routes) {
    return function (constructor) {
        return class extends constructor {
            constructor(...args) {
                super(...args);
                appFunctionSource.routes.call(this, routes);
            }
        };
    };
};
//# sourceMappingURL=decorators.js.map