const events = {
	back    : [],
	forward : [],
	default : [],
};

/**
 * Determines the history direction by looking at the '#' in the url and invokes the fitting callback
 * @param onBack {Function}
 * @param onForward {Function}
 */
const detectBackOrForward = function( onBack: Function, onForward: Function, onDefault: Function )
{
	const hashHistory = [ window.location.hash ];
	let historyLength = window.history.length;

	/* Function to be later invoked as callback */
	return function()
	{
		const sHash  = window.location.hash;
		const length = window.history.length;

		if( hashHistory.length && historyLength === length )
		{
			/* History back */
			if( hashHistory[ hashHistory.length - 2 ] === sHash )
			{
				hashHistory.pop();
				onBack();
			}
			/* History forward */
			else
			{
				hashHistory.push( sHash );
				onForward();
			}
		}
		else
		{
			hashHistory.push( sHash );
			historyLength = length;

			onDefault();
		}
	};
};

export const PitayaNavigator = {
	/**
	 * Registers an event handler to be invoked on history.back/forward
	 * @param eventName {string}
	 * @param callback {Function}
	 */
	on( eventName: string, callback: ( event: Event ) => void )
	{
		if( !( eventName in events ) )
		{
			throw new Error( `Navigation: [ ${eventName} ] is not a valid event - choose one of the following: 'back' 'forward' 'default'.` );
		}

		events[ eventName ].push( callback );
	}
};

/**
 * @callback Handles back navigation
 * @param event {Event}
 */
const navigateBackCallback = function( event )
{
	events.back.forEach( callback => callback( event ) );
};

/**
 * @callback Handles forward navigation
 * @param event {Event}
 */
const navigateForwardCallback = function( event )
{
	events.forward.forEach( callback => callback( event ) );
};

/**
 * @callback Handles default navigation
 * @param event {string}
 */
const navigateDefaultCallback = function( event )
{
	events.default.forEach( callback => callback( event ) );
};

window.addEventListener(
	"hashchange",
	detectBackOrForward(
		navigateBackCallback,
		navigateForwardCallback,
		navigateDefaultCallback
	)
);