"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const aurelia_router_1 = require("aurelia-router");
const app_1 = require("./app");
const app_configuration_1 = require("./app-configuration");
const logger_1 = require("./logger");
const _CLASS_ANIMATION_READY = "pitaya-view--animation-ready";
const _VIEW_ELEMENT_NAME = "pitaya-view";
const _SUB_VIEW_ELEMENT_NAME = "pitaya-sub-view";
const _TABS_VIEW_ELEMENT_NAME = "pitaya-tabs-view";
const _PITAYA_PAGES_CLASS = "au-leave-active";
const _VIEW_STATUS_PRESET = {
    active: false,
    created: false,
    bound: false,
    attached: false,
    animated: false,
    detached: false
};
const _MDC_COLOR_VARIABLE_MAP = {
    "primary": "--mdc-theme-primary",
    "secondary": "--mdc-theme-secondary",
    "on-primary": "--mdc-theme-on-primary",
    "on-secondary": "--mdc-theme-on-secondary",
    "background": "--mdc-theme-background",
    "on-surface": "--mdc-theme-on-surface",
    "surface": "--mdc-theme-surface",
};
let _currentViewInstance = null;
class PitayaView {
    constructor() {
        this._animationTaskQueue = [];
        this._settings = {
            isPublic: true,
            hierarchy: 0,
            theme: app_configuration_1.PitayaAppConfiguration.View.defaultTheme
        };
        this._states = Object.assign({}, _VIEW_STATUS_PRESET);
    }
    static get instance() {
        return _currentViewInstance;
    }
    get rootElement() {
        return this._rootElement;
    }
    get states() {
        return this._states;
    }
    get innerElement() {
        return this._innerElement;
    }
    canActivate(params, routeConfig, navigationInstruction) {
        const { View, Development } = app_configuration_1.PitayaAppConfiguration;
        if (this._settings.isPublic
            || (Development.enabled && Development.disablePrivateRedirects)
            || !View.privateRedirectView) {
            return true;
        }
        return new aurelia_router_1.RedirectToRoute(View.privateRedirectView);
    }
    activate(params, routeConfig, navigationInstruction) {
        this._settings.hierarchy = routeConfig.nav || 0;
        this._states.active = true;
    }
    created(parentContext, context) {
        this._states.created = true;
        const model = context.controllers.find((controller) => {
            const { elementName } = controller.behavior;
            return elementName === _VIEW_ELEMENT_NAME
                || elementName === _SUB_VIEW_ELEMENT_NAME
                || elementName === _TABS_VIEW_ELEMENT_NAME;
        });
        this._viewModel = model.viewModel;
        _currentViewInstance = this;
    }
    bind(bindingContext, overrideContext) {
        this._states.bound = true;
    }
    attached() {
        this._states.attached = true;
        this._rootElement = this._viewModel.rootElement;
        this._innerElement = this._viewModel.innerElement;
        this._setAnimationListener();
    }
    detached() {
        this._states.attached = false;
        this._states.detached = true;
    }
    animated() { }
    async navigate(route, options) {
        return app_1.navigate(route, options);
    }
    queueAnimationTask(task) {
        this._animationTaskQueue.push(task);
    }
    setTheme(color = this._settings.theme) {
        app_1.setStatusBarColor(_getColorFromMdcVariable(color));
    }
    _willBeAnimated() {
        const { rootElement } = this;
        const siblings = [...rootElement.parentElement.children];
        return siblings.some((sibling) => sibling !== rootElement && sibling.classList.contains(_PITAYA_PAGES_CLASS));
    }
    async _setAnimationListener() {
        const { rootElement } = this;
        if (!this._willBeAnimated()) {
            this._onAnimationEnd();
            return;
        }
        rootElement.addEventListener("animationend", this._onAnimationEnd.bind(this), { once: true });
    }
    async _onAnimationEnd() {
        const queue = this._animationTaskQueue;
        while (queue.length) {
            await queue.pop()();
        }
        this.rootElement.classList.add(_CLASS_ANIMATION_READY);
        this._states.animated = true;
        this.animated();
    }
}
exports.PitayaView = PitayaView;
exports.scrollToTop = function (scrollDuration) {
    const scrollStep = -window.scrollY / (scrollDuration / 15);
    const scrollInterval = setInterval(function () {
        if (window.scrollY !== 0) {
            window.scrollBy(0, scrollStep);
        }
        else {
            clearInterval(scrollInterval);
        }
    }, 15);
};
exports.setScrollMarkerHandler = function ({ element, className, minHeight, maxHeight }) {
    const body = document.body;
    const documentElement = document.documentElement;
    const viewHeight = Math.max(documentElement.clientHeight, window.innerHeight || 0);
    minHeight = minHeight === undefined ? 0 : minHeight;
    maxHeight = maxHeight === undefined ? viewHeight : maxHeight;
    const setScrollMarkerClass = function () {
        const scrollTop = Math.max(body.scrollTop, documentElement.scrollTop);
        const addOrRemove = scrollTop >= minHeight && scrollTop <= maxHeight ? "add" : "remove";
        element.classList[addOrRemove](className);
    };
    setScrollMarkerClass();
    window.addEventListener("scroll", setScrollMarkerClass);
    return () => {
        window.removeEventListener("scroll", setScrollMarkerClass);
    };
};
const _getColorFromMdcVariable = function (color) {
    if (color.startsWith("#") || color.startsWith("rgb")) {
        return color;
    }
    if (!(color in _MDC_COLOR_VARIABLE_MAP)) {
        logger_1.log.error("view.js _getColorFromMdcVariable()", `No MDC variable found that matches '${color}'`);
        return "";
    }
    return getComputedStyle(app_1.PitayaApp.instance.appView.rootElement).getPropertyValue(_MDC_COLOR_VARIABLE_MAP[color]);
};
//# sourceMappingURL=view.js.map